/**
 * 
 */
package recursivePossibilitiesStringMVC;

import java.awt.event.*;
import java.util.*;

/**
 * @author deanhougen
 *
 */

public class Model {
	private String baseString;

	private List<String> combinations = new ArrayList<String>();
	private List<String> possibilities = new ArrayList<String>();

	/** Utility field used by event firing mechanism. */
	private ArrayList<ActionListener> actionListenerList;

	private void permutation(String startString, String endString) {
		int n = endString.length();
		if (n == 0) {
			possibilities.add(startString);
		} else {
			for (int i = 0; i < n; i++)
				permutation(startString + endString.charAt(i),
						endString.substring(0, i) + endString.substring(i + 1, n));
		}
	}

	private void comboPermutation(String startString, String endString) {
		if (endString.length() > 0) {
			String newCombo = startString + endString.charAt(0);
			combinations.add(newCombo);
			permutation("", newCombo);
			comboPermutation(newCombo, endString.substring(1));
			comboPermutation(startString, endString.substring(1));
		}
	}

	/**
	 * @return the baseString
	 */
	public String getBaseString() {
		return baseString;
	}

	/**
	 * @param baseString
	 *            the baseString to set
	 */
	public void setBaseString(String baseString) {
		this.baseString = baseString;

		comboPermutation("", baseString);

		// Notify the listener for the change in base string
		processEvent(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "base string"));
	}

	/**
	 * @return the combinations
	 */
	public List<String> getCombinations() {
		return combinations;
	}

	/**
	 * @return the possibilities
	 */
	public List<String> getPossibilities() {
		return possibilities;
	}

	/** Register an action event listener */
	public synchronized void addActionListener(ActionListener l) {
		if (actionListenerList == null)
			actionListenerList = new ArrayList<ActionListener>();

		actionListenerList.add(l);
	}

	/** Remove an action event listener */
	public synchronized void removeActionListener(ActionListener l) {
		if (actionListenerList != null && actionListenerList.contains(l))
			actionListenerList.remove(l);
	}

	/** Fire Event */
	private void processEvent(ActionEvent e) {
		ArrayList<ActionListener> list;

		synchronized (this) {
			if (actionListenerList == null)
				return;
			list = (ArrayList<ActionListener>) actionListenerList.clone();
		}

		for (int i = 0; i < list.size(); i++) {
			ActionListener listener = list.get(i);
			listener.actionPerformed(e);
		}
	}
}